@@ -1,7 +1,6 @@
-import json
 import os
 import sys
-from .config import HOME, EXPROOT, CACHEROOT
+from .config import HOME, EXP_ROOT, CACHE_ROOT, BLOB_ROOT
 
 
 def home():
@@ -11,12 +10,9 @@ def home():
 
 
 def cache_dir():
-    if CACHEROOT:
-        return CACHEROOT
-
     try:
-        res = os.path.join(home(), '.cache/lumo')
-        os.makedirs(res, exist_ok=True)
+        os.makedirs(CACHE_ROOT, exist_ok=True)
+        res = CACHE_ROOT
     except PermissionError:
         res = os.path.join(home(), '.lumo/.cache')
         os.makedirs(res, exist_ok=True)
@@ -37,8 +33,14 @@ def libhome():
 
 
 def exproot():
-    if EXPROOT:
-        return EXPROOT
+    if EXP_ROOT:
+        return EXP_ROOT
+    return libhome()
+
+
+def blobroot():
+    if BLOB_ROOT:
+        return BLOB_ROOT
     return libhome()
 
 
