@@ -293,12 +293,12 @@ class EvalCallback(TrainCallback):
 
     def _test_or_eval(self, params: ParamsType, trainer: Trainer):
         if self.has_eval:
-            if params.eidx % self.eval_in_per_epoch == self.eval_in_per_epoch - 1:
-                self._last_eval = params.eidx
+            if trainer.eidx % self.eval_in_per_epoch == self.eval_in_per_epoch - 1:
+                self._last_eval = trainer.eidx
                 trainer.evaluate()
         if self.has_test:
-            if params.eidx % self.test_in_per_epoch == self.test_in_per_epoch - 1:
-                self._last_test = params.eidx
+            if trainer.eidx % self.test_in_per_epoch == self.test_in_per_epoch - 1:
+                self._last_test = trainer.eidx
                 trainer.test()
 
     def on_train_epoch_end(self, trainer: Trainer, func, params: ParamsType, record: Record, *args,
@@ -306,9 +306,9 @@ class EvalCallback(TrainCallback):
         self._test_or_eval(params, trainer)
 
     def on_train_end(self, trainer: Trainer, func, params: ParamsType, record: Record, *args, **kwargs):
-        if self._last_eval != params.eidx and self.has_eval:
+        if self._last_eval != trainer.eidx and self.has_eval:
             trainer.evaluate()
-        if self._last_test != params.eidx and self.has_test:
+        if self._last_test != trainer.eidx and self.has_test:
             trainer.test()
 
     def __repr__(self):
@@ -324,13 +324,6 @@ class DebugCallback(BaseCallback):
     def on_first_exception(self, source: Trainer, func, params: ParamsType, e: BaseException, *args, **kwargs):
         super().on_first_exception(source, func, params, e, *args, **kwargs)
         print('on_first_exception()', func.__name__)
-        # from rich.console import Console
-        # console = Console()
-        # try:
-        #     import six
-        #     six.reraise(type(e), e, e.__traceback__)
-        #
-        # except:
 
     def on_exception(self, source: Trainer, func, params: ParamsType, e: BaseException, *args, **kwargs):
         return super().on_exception(source, func, params, e, *args, **kwargs)
@@ -349,6 +342,7 @@ class DebugCallback(BaseCallback):
 
 
 class LoggerCallback(TrainCallback, InitialCallback):
+    priority = 99999
 
     def __init__(self, step_frequence=3, break_in=1000):
         self.stage = {}
@@ -384,7 +378,7 @@ class LoggerCallback(TrainCallback, InitialCallback):
 
     def on_train_begin(self, trainer: Trainer, func, params: ParamsType, *args, **kwargs):
         super().on_train_begin(trainer, func, params, *args, **kwargs)
-        trainer.logger.info('[[Train Begin]]')
+        trainer.logger.raw('[[Train Begin]]')
         self.global_tqdm = inlinetqdm(total=params.epoch,
                                       position=0, leave=True,
                                       bar_format='Ela: {elapsed} | Last: {remaining} | Avg: {rate}',
@@ -421,12 +415,12 @@ class LoggerCallback(TrainCallback, InitialCallback):
     def on_test_begin(self, trainer: Trainer, func, params: ParamsType, *args, **kwargs):
         super().on_test_begin(trainer, func, params, *args, **kwargs)
         self.renew(TrainStage.test)
-        trainer.logger.info('[[Test Begin]]')
+        trainer.logger.raw('[[Test Begin]]')
 
     def on_eval_begin(self, trainer: Trainer, func, params: ParamsType, *args, **kwargs):
         super().on_eval_begin(trainer, func, params, *args, **kwargs)
         self.renew(TrainStage.val)
-        trainer.logger.info('[[Evaluate Begin]]')
+        trainer.logger.raw('[[Evaluate Begin]]')
 
     @staticmethod
     def format_interval(t: float):
@@ -465,29 +459,29 @@ class LoggerCallback(TrainCallback, InitialCallback):
         last = time.time() - self.time
         last_str = self.format_interval(last)
 
-        return f'{n}/{total} train: {elapsed_str} | epoch: {last_str} | last: {remaining_str} | avg: {rate_str}'
+        return f'[Epoch {n}/{total} End] cost: {elapsed_str} | current: {last_str} | remain: {remaining_str} | avg: {rate_str}'
 
     def on_train_epoch_end(self, trainer: Trainer, func, params: ParamsType, record: Record, *args, **kwargs):
         super().on_train_epoch_end(trainer, func, params, record, *args, **kwargs)
         self.global_tqdm.update()
         # self.flush(trainer)
-        trainer.logger.info(self.format_train_epoch_time(**self.global_tqdm.format_dict))
+        trainer.logger.raw(self.format_train_epoch_time(**self.global_tqdm.format_dict))
 
     def on_test_end(self, trainer: Trainer, func, params: ParamsType,
                     record: Optional[Record] = None,
                     *args, **kwargs):
         self.flush(trainer)
-        trainer.logger.info('[[Test End]]')
+        trainer.logger.raw('[[Test End]]')
 
     def on_eval_end(self, trainer: Trainer, func, params: ParamsType,
                     record: Optional[Record] = None,
                     *args, **kwargs):
         self.flush(trainer)
-        trainer.logger.info('[[Evaluate End]]')
+        trainer.logger.raw('[[Evaluate End]]')
 
     def on_train_end(self, trainer: Trainer, func, params: ParamsType, record: Optional[Record], *args, **kwargs):
         self.flush(trainer)
-        trainer.logger.info('[[Train End]]')
+        trainer.logger.raw('[[Train End]]')
 
     def on_train_step_end(self, trainer: Trainer, func, params: ParamsType,
                           metric: MetricType = None,
@@ -526,7 +520,7 @@ class EpochCheckpoint(TrainCallback):
     def on_train_epoch_end(self, trainer: Trainer, func, params: ParamsType, record: Optional[Record], *args,
                            **kwargs):
         meter = record.avg()
-        if params.eidx % self.per_epoch == 0 and params.eidx > 0:
+        if trainer.eidx % self.per_epoch == 0 and trainer.eidx > 0:
             trainer.save_checkpoint(meta_info=Meter.wrap_result(meter))
 
     def __repr__(self) -> str:
@@ -541,7 +535,7 @@ class GlobalStepCheckpoint(TrainCallback):
 
     def on_train_step_end(self, trainer: Trainer, func, params: ParamsType, metric: Meter, *args, **kwargs):
         super().on_train_step_end(trainer, func, params, metric, *args, **kwargs)
-        if params.global_step % self.per == 0 and params.global_step > 0:
+        if trainer.global_steps % self.per == 0 and trainer.global_steps > 0:
             trainer.save_checkpoint(meta_info=Meter.wrap_result(metric))
 
 
