@@ -32,6 +32,27 @@ from .saver import Saver
 
 
 class Trainer(_BaseTrainer):
+    """
+    Trainer provides a complete training and evaluation logic to help users focus on specific training details.
+    It integrates logging, metric recording, experiment saving, version control, and complete callback control
+    of each life cycle.
+
+    When to use lumo.Trainer:
+     - At the beginning of the investigation direction, there will be many code branches and experiments
+        based on the same set of data and models, but with different training details.
+     - Reproduce multiple papers in the same field, which are also based on the same dataset and evaluation logic.
+
+    When to simply use lumo.SimpleExperiment:
+     - You have maintained a mature training framework yourself, but there are still certain requirements
+        for allocating dynamic(different) storage path and version control in each program.
+
+    At present, I have explored best practices based on lumo in two fields, this includs:
+     - https://github.com/pytorch-lumo/image-classification
+     - https://github.com/pytorch-lumo/MMERC
+
+    Both two reposiroties are github templates and you can use them to create you new (private) repository.
+    """
+
     callback_function = {
         "save_keypoint", "save_checkpoint", "save_model", "load_state_dict",
         'ioptims', 'imodels', 'train', 'test', 'evaluate', 'train_epoch', 'train_step',
@@ -74,7 +95,9 @@ class Trainer(_BaseTrainer):
         if dist.is_main():
             self.params.to_yaml(self.exp.test_file('params.yaml'))
 
-        self.set_global_steps(params.get('global_steps', 0))
+        self.set_global_steps(0)
+        self.set_epoch_idx(0)
+        self.set_idx(0)
         if params.get('debug', False):
             self.exp.set_prop('debug', True)
 
@@ -153,33 +176,33 @@ class Trainer(_BaseTrainer):
         ------
         When using add_embedding, there may raise some exceptions cased by version conflict, here is some solutions:
 
-        1. tensorflow_core._api.v1.io.gfile' or'tensorflow_core._api.v2.io.gfile' has no attribute 'get_filesystem'
+        1. `tensorflow_core._api.v1.io.gfile` or `tensorflow_core._api.v2.io.gfile` has no attribute `get_filesystem`
         first, try upgrade tensorboard and tensorflow as followed version:
             tensorboard==2.0.2
             tensorflow==2.0.0
 
-        if you still have the same problem, use this code as a temporary solution:
+        If you still have the same problem, use this code as a temporary solution:
 
             import tensorflow as tf
             import tensorboard as tb
             tf.io.gfile = tb.compat.tensorflow_stub.io.gfile
 
-        use `trainer.safe_writer` to get a writter with these code added inside thexp.
+        Use `trainer.safe_writer` to get a writter with these code added inside thexp.
 
-        solution is referred by https://github.com/pytorch/pytorch/issues/30966
+        > This solution is referred by https://github.com/pytorch/pytorch/issues/30966 .
 
 
-        2. You may cause PermissionError like: [Errno 13] Permission denied: '/tmp/.tensorboard-info/pid-20281.info'
-        the solution is to set environment variable TMPDIR
+        2. You may cause PermissionError like: [Errno 13] Permission denied: '/tmp/.tensorboard-info/pid-20281.info'.
+        The solution is to set environment variable TMPDIR
 
             export TMPDIR=/tmp/$USER;
             mkdir -p $TMPDIR;
             tensorboard --logdir ...
 
-        code in line:
+        One line command version:
             export TMPDIR=/tmp/$USER; mkdir -p $TMPDIR; tensorboard --logdir ....
 
-        solution is referred by https://github.com/tensorflow/tensorboard/issues/2010
+        > This solution is referred by https://github.com/tensorflow/tensorboard/issues/2010 .
 
         Returns:
             A SummaryWriter instance
@@ -187,7 +210,10 @@ class Trainer(_BaseTrainer):
         try:
             from torch.utils.tensorboard import SummaryWriter
         except ModuleNotFoundError:
-            from tensorboardX import SummaryWriter
+            try:
+                from tensorboardX import SummaryWriter
+            except ModuleNotFoundError:
+                return None
 
         kwargs = self.exp.board_args
         res = SummaryWriter(**kwargs)
@@ -201,20 +227,37 @@ class Trainer(_BaseTrainer):
 
     @property
     def first_epoch(self):
+        """Determine whether the current epoch is the first"""
         return self.eidx == 0
 
     @property
     def first_step(self):
+        """Determine whether the current step is the first"""
         return self.global_steps == 0
 
     @property
     def idx(self):
+        # started from 0
         return self._prop.get('idx', 0)
 
     @property
     def eidx(self):
+        """
+        This value will be automatically incremented by 1 before calling the train_epoch method
+        current epoch round. The value remains unchanged in below methods:
+         - `.on_train_epoch_begin()`: eidx
+         - `.train_step()`: eidx
+         - `.on_train_epoch_end()`: eidx
+
+        This behaviour of `.idx` and `.global_steps` is the same as `.eidx`
+        """
         return self._prop.get('eidx', 0)
 
+    @property
+    def global_steps(self) -> int:
+        # started from 0
+        return self._prop['global_steps']
+
     @property
     def trainer_state(self):
         return self._prop
@@ -230,7 +273,6 @@ class Trainer(_BaseTrainer):
 
     @property
     def optim_dict(self) -> Dict[str, Optimizer]:
-
         return {key: self[key] for key in self._state_dicts['optims']}
 
     @property
@@ -261,10 +303,6 @@ class Trainer(_BaseTrainer):
     def val_dataloader(self) -> Optional[DataLoaderType]:
         return self.datamodule['val']
 
-    @property
-    def global_steps(self) -> int:
-        return self._prop['global_steps']
-
     @property
     def device(self):
         return self.accelerate.device
